use serde::{Deserialize, Serialize};


#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Theme {
    #[serde(default="Theme::default_accent_color")]
    pub accent_color: egui::Color32,
    #[serde(default="Theme::default_primary_bg_color")]
    pub primary_bg_color: egui::Color32,
    #[serde(default="Theme::default_secondary_bg_color")]
    pub secondary_bg_color: egui::Color32,
    #[serde(default="Theme::default_text_color")]
    pub text_color: egui::Color32,
    #[serde(default="Theme::default_dim_text_color")]
    pub dim_text_color: egui::Color32,
}

impl Default for Theme {
    fn default() -> Self {
        Self {
            accent_color: Self::default_accent_color(),
            primary_bg_color: Self::default_primary_bg_color(),
            secondary_bg_color: Self::default_secondary_bg_color(),
            text_color: Self::default_text_color(),
            dim_text_color: Self::default_dim_text_color(),
        }
    }
}

impl Theme {
    fn default_accent_color() -> egui::Color32 {
        egui::Color32::from_rgb(5, 102, 146) // #0566F6
    }
    fn default_primary_bg_color() -> egui::Color32 {
        egui::Color32::from_rgb(31, 34, 40) // #1F2228
    }
    fn default_secondary_bg_color() -> egui::Color32 {
        egui::Color32::from_rgb(47, 53, 61)  // #2f353d
    }
    fn default_text_color() -> egui::Color32 {
        egui::Color32::from_rgb(223, 223, 223)  //  #dfdfdf
    }
    fn default_dim_text_color() -> egui::Color32 {
        egui::Color32::from_rgb(175, 175, 175 )  //  #afafaf
    }

}
